import java.util.*;
import java.io.*;

public class InvertedIndex {
	public static final String[] docs = new String[] {"doc1.txt", "doc2.txt", "doc3.txt"};
	
	public static void main(String[] args) throws Throwable {
		// STEP 1: Index each document
		Map<String, Set<String>> index = new TreeMap<String, Set<String>>();
		
		// Let us index each of the documents
		for (String file : docs) {
			// Get the words for each file
			Set<String> words = readFile(file);
			// Map each word to the file they appear in
			for (String word : words) {
				if (!index.containsKey(word)) {
					index.put(word, new TreeSet<String>());
				}
				index.get(word).add(file);
			}
		}
		
		// Print the inverted index in column form
		for (String word : index.keySet()) {
			System.out.println(word + ": " + index.get(word));
		}
		System.out.println();
		
		// Ask the user for their query
		Scanner in = new Scanner(System.in);
		while (true) {
			System.out.print("Query: ");
			String q = in.nextLine();
			Scanner words = new Scanner(q);
			// get the set of possible files from the first word in the query
			Set<String> files = new TreeSet<String>(index.get(words.next()));
			while (words.hasNext()) {
				// only keep files that contain the other words in the query
				files.retainAll(index.get(words.next()));
			}
			System.out.println(files);
		}
	}
	
	// Reads in a file of the given name and returns the set of words in the file
	public static Set<String> readFile(String name) throws FileNotFoundException {
		Scanner input = new Scanner(new File(name));
		Set<String> words = new TreeSet<String>();
		
		while (input.hasNext()) {
			String next = input.next();
			words.add(next);
		}
		
		return words;
	}
}